/*
 * This file is autogenerated by construct_dictionary.py.
 * Do not edit this file by hand.
 */

#include "src/include/pmix_config.h"
#include "src/include/pmix_globals.h"
#include "include/pmix_common.h"

#ifndef PMIX_DICTIONARY_H
#define PMIX_DICTIONARY_H

BEGIN_C_DECLS

PMIX_EXPORT extern const pmix_regattr_input_t pmix_dictionary[529];

#define PMIX_INDEX_BOUNDARY 528

END_C_DECLS

#endif

