/*
 * Copyright (c) 2016      Mellanox Technologies, Inc.
 *                         All rights reserved.
 * Copyright (c) 2018      IBM Corporation.  All rights reserved.
 * Copyright (c) 2018      Intel, Inc. All rights reserved.
 * Copyright (c) 2019      Research Organization for Information Science
 *                         and Technology (RIST).  All rights reserved.
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */


#ifndef PMIx_VERSION_H
#define PMIx_VERSION_H

/* define PMIx version */
#define PMIX_VERSION_MAJOR 4L
#define PMIX_VERSION_MINOR 2L
#define PMIX_VERSION_RELEASE 2L

#define PMIX_NUMERIC_VERSION 0x00040202
#endif
